import asyncio

from benchmark.benchmarks import (
    bench_python,
    benchmark_aioinject,
    benchmark_dependency_injector,
    benchmark_di,
    benchmark_dishka,
    benchmark_lagom,
    benchmark_punq,
    benchmark_rodi,
)
from benchmark.lib.bench import Benchmark
from benchmark.lib.format import print_markdown_table, print_results


async def main() -> None:
    bench = Benchmark(
        benchmarks=[
            bench_python,
            benchmark_aioinject,
            benchmark_dishka,
            benchmark_rodi,
            benchmark_di,
            benchmark_dependency_injector,
            benchmark_punq,
            benchmark_lagom,
        ]
    )
    results = await bench.run(rounds=[100_000])
    print_results(results)
    print_markdown_table(results)


if __name__ == "__main__":
    asyncio.run(main())



from aioinject import Container, Object, Scoped
from aioinject.testing import TestContainer


async def test_override() -> None:
    container = Container()
    container.register(Scoped(lambda: 0, interface=int))

    testcontainer = TestContainer(container)

    override = 42

    async with (
        container.context() as ctx,
        testcontainer.override(Object(override)),
    ):
        assert await ctx.resolve(int) is override

    async with container.context() as ctx:
        assert await ctx.resolve(int) == 0

    # Test sync override
    async with (
        container.context() as ctx,
    ):
        with testcontainer.override(Object(override)):
            assert await ctx.resolve(int) is override



from __future__ import annotations

from types import TracebackType
from typing import TYPE_CHECKING, Any

from typing_extensions import Self

from aioinject.context import ProviderRecord
from aioinject.extensions.providers import ProviderInfo


if TYPE_CHECKING:
    from aioinject import Container, Provider, SyncContainer


class _Override:
    def __init__(
        self, container: Container | SyncContainer, provider: Provider[object]
    ) -> None:
        self.container = container
        self.registry = container.registry
        self.provider = provider
        self.prev: list[ProviderRecord[Any]] | None = None

        self.extension = self.registry.find_provider_extension(self.provider)
        self.info: ProviderInfo[Any] = self.extension.extract(
            self.provider, self.registry.type_context
        )

    async def __aenter__(self) -> Self:
        self._enter()
        return self

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None:
        self._exit()

    def __enter__(self) -> Self:
        self._enter()
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None:
        self._exit()

    def _enter(self) -> None:
        self._clear()

        self.prev = self.registry.providers.get(self.info.interface)

        self.registry.providers[self.info.interface] = [
            ProviderRecord(
                provider=self.provider,
                ext=self.extension,
                info=self.info,
            )
        ]

    def _exit(self) -> None:
        if self.prev is not None:
            self.registry.providers[self.info.interface] = self.prev

        self._clear()

    def _clear(self) -> None:
        self.registry.compilation_cache.clear()
        self.container.root.cache.clear()


class TestContainer:
    __test__ = False  # pytest

    def __init__(self, container: Container | SyncContainer) -> None:
        self.container = container

    def override(self, provider: Provider[Any]) -> _Override:
        return _Override(self.container, provider)



import dependency_injector.containers
import dependency_injector.providers
import di
import di.executors
import dishka
import lagom
import punq  # type: ignore[import-untyped]
import rodi
from di import bind_by_type
from di.dependent import Dependent

import aioinject
from aioinject import Scoped
from benchmark.dependencies import (
    RepositoryA,
    RepositoryB,
    ServiceA,
    ServiceB,
    Session,
    UseCase,
    create_session,
    create_session_cm,
)
from benchmark.lib.bench import BenchmarkContext, bench


@bench(name="aioinject")
async def benchmark_aioinject(context: BenchmarkContext) -> None:
    container = aioinject.Container()
    container.register(Scoped(create_session_cm))
    container.register(Scoped(RepositoryA))
    container.register(Scoped(RepositoryB))
    container.register(Scoped(ServiceA))
    container.register(Scoped(ServiceB))
    container.register(Scoped(UseCase))

    async with container.context() as ctx:
        await ctx.resolve(UseCase)

    for _ in range(context.rounds):
        with context.round():
            async with container.context() as ctx:
                await ctx.resolve(UseCase)


@bench(name="dishka")
async def benchmark_dishka(context: BenchmarkContext) -> None:
    provider = dishka.Provider(scope=dishka.Scope.REQUEST)
    provider.provide(create_session)
    provider.provide(RepositoryA)
    provider.provide(RepositoryB)
    provider.provide(ServiceA)
    provider.provide(ServiceB)
    provider.provide(UseCase)
    container = dishka.make_async_container(provider)

    async with container() as ctx:
        await ctx.get(UseCase)

    for _ in range(context.rounds):
        with context.round():
            async with container() as ctx:
                await ctx.get(UseCase)


@bench(name="python")
async def bench_python(context: BenchmarkContext) -> None:
    for _ in range(context.rounds):
        with context.round():
            async with create_session_cm() as session:
                repo_a = RepositoryA(session=session)
                repo_b = RepositoryB(session=session)
                svc_a = ServiceA(repository=repo_a)
                svc_b = ServiceB(repository=repo_b)
                UseCase(service_a=svc_a, service_b=svc_b)


@bench(name="rodi")
async def benchmark_rodi(context: BenchmarkContext) -> None:
    container = rodi.Container()
    container.add_scoped(Session)
    container.add_scoped(RepositoryA)
    container.add_scoped(RepositoryB)
    container.add_scoped(ServiceA)
    container.add_scoped(ServiceB)
    container.add_scoped(UseCase)

    container.resolve(UseCase)

    for _ in range(context.rounds):
        with context.round():
            container.resolve(UseCase)


@bench(name="adriangb/di")
async def benchmark_di(context: BenchmarkContext) -> None:
    container = di.Container()
    container.bind(
        bind_by_type(Dependent(create_session, scope="request"), Session)
    )
    solved = container.solve(
        Dependent(UseCase, scope="request"), scopes=["request"]
    )

    executor = di.executors.AsyncExecutor()
    async with container.enter_scope("request") as state:
        await solved.execute_async(executor=executor, state=state)

    for _ in range(context.rounds):
        with context.round():
            async with container.enter_scope("request") as state:
                await solved.execute_async(executor=executor, state=state)


@bench(name="dependency-injector")
async def benchmark_dependency_injector(context: BenchmarkContext) -> None:
    class Container(dependency_injector.containers.DeclarativeContainer):
        session = dependency_injector.providers.Factory(Session)
        repository_a = dependency_injector.providers.Factory(
            RepositoryA, session=session
        )
        repository_b = dependency_injector.providers.Factory(
            RepositoryB, session=session
        )
        service_a = dependency_injector.providers.Factory(
            ServiceA, repository=repository_a
        )
        service_b = dependency_injector.providers.Factory(
            ServiceB, repository=repository_b
        )
        use_case = dependency_injector.providers.Factory(
            UseCase, service_a=service_a, service_b=service_b
        )

    container = Container()

    container.use_case()

    for _ in range(context.rounds):
        with context.round():
            container.use_case()


@bench(name="punq", max_iterations=5_000)
async def benchmark_punq(context: BenchmarkContext) -> None:
    container = punq.Container()
    container.register(Session)
    container.register(RepositoryA)
    container.register(RepositoryB)
    container.register(ServiceA)
    container.register(ServiceB)
    container.register(UseCase)

    container.resolve(UseCase)

    for _ in range(context.rounds):
        with context.round():
            container.resolve(UseCase)


@bench(name="lagom")
async def benchmark_lagom(context: BenchmarkContext) -> None:
    # Note: not using lifetimes here with lagom

    container = lagom.Container()

    for _ in range(context.rounds):
        with context.round():
            container[UseCase]



from typing import Any

from aiogram import Router

import aioinject
from aioinject import Container, Object
from aioinject.ext.aiogram import AioInjectMiddleware


_NUMBER = 42


async def test_middleware() -> None:
    container = Container()
    container.register(Object(_NUMBER))

    middleware = AioInjectMiddleware(container=container)
    event_ = object()
    data_: dict[str, Any] = {}

    async def handler(
        event: object,
        data: dict[str, Any],
    ) -> None:
        assert event is event_
        assert data is data_
        assert isinstance(data["aioinject_context"], aioinject.Context)

    await middleware(handler=handler, event=event_, data=data_)  # type: ignore[arg-type]


def test_add_to_router() -> None:
    middleware = AioInjectMiddleware(container=Container())

    router = Router()
    middleware.add_to_router(router=router)

    for observer in router.observers.values():
        assert observer.outer_middleware[0] is middleware
